==> currents-api-types-0.0.19.tgz: package/utils/string/entropy.ts <==
// mostly taken from https://github.com/mvhenten/string-entropy, MIT license
import { values } from '../../utils/object';

const LOWERCASE_ALPHA = 'abcdefghijklmnopqrstuvwxyz',
  UPPERCASE_ALPHA = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
  DIGITS = '0123456789',
  PUNCT1 = '!@#$%^&*()',
  PUNCT2 = '~`-_=+[]{}\\|;:\'",.<>?/';

// Calculate the size of the alphabet.
//
// This is a mostly back-of-the hand calculation of the alphabet.
// We group the a-z, A-Z and 0-9 together with the leftovers of the keys on an US keyboard.
// Characters outside ascii add one more to the alphabet. Meaning that the alphabet size of the word:
// "ümlout" will yield 27 characters. There is no scientific reasoning behind this, besides to
// err on the save side.
/**
 * @param {Str} s String to calculate the alphabet from
 * @returns {Number} n Size of the alphabet
 */
function alphabetSize(s: string) {
  const collect = {
    unicode: 0,
    size: 0,
    alpha: 0,
    alcaps: 0,
    digits: 0,
    punct1: 0,
  };

  for (let i = 0; i < s.length; i++) {
    const c = s[i];

    // we only need to look at each character once
    if (s.indexOf(c) !== i) continue;
    if (LOWERCASE_ALPHA.indexOf(c) !== -1) collect.alpha = LOWERCASE_ALPHA.length;
    else if (UPPERCASE_ALPHA.indexOf(c) !== -1) collect.alcaps = UPPERCASE_ALPHA.length;
    else if (DIGITS.indexOf(c) !== -1) collect.digits = DIGITS.length;
    else if (PUNCT1.indexOf(c) !== -1) collect.punct1 = PUNCT1.length;
    else if (PUNCT2.indexOf(c) !== -1) collect.size = PUNCT2.length;
    // the original lib author can only guess the size of a non-western alphabet.
    // The choice here is to grant the size of the western alphabet, together
    // with an additional bonus for the character itself.
    //
    else if (c.charCodeAt(0) > 127) {
      collect.alpha = 26;
      collect.unicode += 1;
    }
  }

  return values(collect).reduce((a, b) => a + b, 0);
}

// Calculate [information entropy](https://en.wikipedia.org/wiki/Password_strength#Entropy_as_a_measure_of_password_strength)
/**
 * @param {String} s String to calculate entropy for
 * @returns {Number} entropy
 */
export const entropy = (s: string) => {
  return Math.round(s.length * (Math.log(alphabetSize(s)) / Math.log(2)));
};

==> currents-api-types-0.0.77.tgz: package/utils/string/entropy.ts <==
// mostly taken from https://github.com/mvhenten/string-entropy, MIT license
import { values } from '../object';

const LOWERCASE_ALPHA = 'abcdefghijklmnopqrstuvwxyz',
  UPPERCASE_ALPHA = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
  DIGITS = '0123456789',
  PUNCT1 = '!@#$%^&*()',
  PUNCT2 = '~`-_=+[]{}\\|;:\'",.<>?/';

// Calculate the size of the alphabet.
//
// This is a mostly back-of-the hand calculation of the alphabet.
// We group the a-z, A-Z and 0-9 together with the leftovers of the keys on a US keyboard.
// Characters outside ascii add one more to the alphabet. Meaning that the alphabet size of the word:
// "ümlout" will yield 27 characters. There is no scientific reasoning behind this, besides to
// err on the save side.
function alphabetSize(s: string) {
  const collect = {
    unicode: 0,
    size: 0,
    alpha: 0,
    alcaps: 0,
    digits: 0,
    punct1: 0,
  };

  for (let i = 0; i < s.length; i++) {
    const c = s[i];

    // we only need to look at each character once
    if (s.indexOf(c) !== i) continue;
    if (LOWERCASE_ALPHA.indexOf(c) !== -1) collect.alpha = LOWERCASE_ALPHA.length;
    else if (UPPERCASE_ALPHA.indexOf(c) !== -1) collect.alcaps = UPPERCASE_ALPHA.length;
    else if (DIGITS.indexOf(c) !== -1) collect.digits = DIGITS.length;
    else if (PUNCT1.indexOf(c) !== -1) collect.punct1 = PUNCT1.length;
    else if (PUNCT2.indexOf(c) !== -1) collect.size = PUNCT2.length;
    // the original lib author can only guess the size of a non-western alphabet.
    // The choice here is to grant the size of the western alphabet, together
    // with an additional bonus for the character itself.
    //
    else if (c.charCodeAt(0) > 127) {
      collect.alpha = 26;
      collect.unicode += 1;
    }
  }

  return values(collect).reduce((a, b) => a + b, 0);
}

// Calculate [information entropy](https://en.wikipedia.org/wiki/Password_strength#Entropy_as_a_measure_of_password_strength)
export const entropy = (s: string) => Math.round(s.length * (Math.log(alphabetSize(s)) / Math.log(2)));
